package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.List;
import java.util.Objects;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

final class SelectorComponentImpl extends AbstractComponent implements SelectorComponent {
   private final String pattern;
   private final @Nullable Component separator;

   static SelectorComponent create(final @NotNull List<? extends ComponentLike> children, final @NotNull Style style, final @NotNull String pattern, final @Nullable ComponentLike separator) {
      return new SelectorComponentImpl(ComponentLike.asComponents(children, IS_NOT_EMPTY), (Style)Objects.requireNonNull(style, "style"), (String)Objects.requireNonNull(pattern, "pattern"), ComponentLike.unbox(separator));
   }

   SelectorComponentImpl(final @NotNull List<Component> children, final @NotNull Style style, final @NotNull String pattern, final @Nullable Component separator) {
      super(children, style);
      this.pattern = pattern;
      this.separator = separator;
   }

   public @NotNull String pattern() {
      return this.pattern;
   }

   public @NotNull SelectorComponent pattern(final @NotNull String pattern) {
      return (SelectorComponent)(Objects.equals(this.pattern, pattern) ? this : create(this.children, this.style, pattern, this.separator));
   }

   public @Nullable Component separator() {
      return this.separator;
   }

   public @NotNull SelectorComponent separator(final @Nullable ComponentLike separator) {
      return create(this.children, this.style, this.pattern, separator);
   }

   public @NotNull SelectorComponent children(final @NotNull List<? extends ComponentLike> children) {
      return create(children, this.style, this.pattern, this.separator);
   }

   public @NotNull SelectorComponent style(final @NotNull Style style) {
      return create(this.children, style, this.pattern, this.separator);
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (!(other instanceof SelectorComponent)) {
         return false;
      } else if (!super.equals(other)) {
         return false;
      } else {
         SelectorComponent that = (SelectorComponent)other;
         return Objects.equals(this.pattern, that.pattern()) && Objects.equals(this.separator, that.separator());
      }
   }

   public int hashCode() {
      int result = super.hashCode();
      result = 31 * result + this.pattern.hashCode();
      result = 31 * result + Objects.hashCode(this.separator);
      return result;
   }

   public String toString() {
      return Internals.toString(this);
   }

   @NotNull
   public SelectorComponent.@NotNull Builder toBuilder() {
      return new BuilderImpl(this);
   }

   static final class BuilderImpl extends AbstractComponentBuilder<SelectorComponent, SelectorComponent.Builder> implements SelectorComponent.Builder {
      private @Nullable String pattern;
      private @Nullable Component separator;

      BuilderImpl() {
      }

      BuilderImpl(final @NotNull SelectorComponent component) {
         super(component);
         this.pattern = component.pattern();
         this.separator = component.separator();
      }

      @NotNull
      public SelectorComponent.@NotNull Builder pattern(final @NotNull String pattern) {
         this.pattern = (String)Objects.requireNonNull(pattern, "pattern");
         return this;
      }

      @NotNull
      public SelectorComponent.@NotNull Builder separator(final @Nullable ComponentLike separator) {
         this.separator = ComponentLike.unbox(separator);
         return this;
      }

      public @NotNull SelectorComponent build() {
         if (this.pattern == null) {
            throw new IllegalStateException("pattern must be set");
         } else {
            return SelectorComponentImpl.create(this.children, this.buildStyle(), this.pattern, this.separator);
         }
      }
   }
}
